package org.expeditee.io;

import java.awt.Dimension;
import java.awt.Image;
import java.awt.Polygon;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.Writer;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import org.expeditee.gui.Frame;
import org.expeditee.gui.FrameGraphics;
import org.expeditee.items.Circle;
import org.expeditee.items.Item;
import org.expeditee.items.Line;
import org.expeditee.items.Picture;
import org.expeditee.items.Text;
import org.expeditee.settings.UserSettings;

import com.lowagie.text.Document;
import com.lowagie.text.DocumentException;
import com.lowagie.text.Font;
import com.lowagie.text.FontFactory;
import com.lowagie.text.Rectangle;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfWriter;

/**
 * 
 * @author root
 *
 */
public class PDF2Writer extends DefaultFrameWriter {

	private Document _pdfDocument;

	private PdfWriter _pdfWriter;

	public PDF2Writer() {
		Dimension d = FrameGraphics.getMaxSize();
		_pdfDocument = new Document(new Rectangle(d.width, d.height));
	}

	@Override
	protected void initialise(Frame start, Writer writer) throws IOException {
		_format = ".pdf";
		super.initialise(start, writer);
		try {
			_pdfWriter = PdfWriter.getInstance(_pdfDocument,
					new FileOutputStream(_filename));
			_pdfDocument.open();
			_pdfDocument.addCreationDate();
			_pdfDocument.addAuthor(UserSettings.UserName.get());
			_pdfDocument.addCreator("Expeditee");
		} catch (DocumentException e) {
			e.printStackTrace();
			throw new IOException(e.getMessage());
		}
	}

	@Override
	protected void writeTitle(Text toWrite, List<Item> items)
			throws IOException {
		_pdfDocument.addTitle(toWrite.getText());
		writeText(toWrite);
	}

	@Override
	protected String finalise() throws IOException {
		_pdfDocument.close();
		return super.finalise();
	}

	@Override
	protected void writeText(Text text) throws IOException {
		PdfContentByte cb = _pdfWriter.getDirectContent();
		// we tell the ContentByte we're ready to draw text
		cb.beginText();
		Font font = FontFactory.getFont(
				Conversion.getPdfFont(text.getFamily()), text.getSize(), text
						.getPaintFont().getStyle(), text.getPaintColor());
		cb.setFontAndSize(font.getBaseFont(), text.getSize());
		// cb.setColorStroke(text.getPaintColor());
		cb.setColorFill(text.getPaintColor());

		// we draw some text on a certain position
		cb.setTextMatrix(text.getX(), _pdfWriter.getPageSize().getHeight() - text.getY());
		List<String> textLines = text.getTextList();
		//cb.showText(textLines.get(0));
		for(int i = 0; i < textLines.size(); i++){
			cb.showText(textLines.get(i));
			cb.moveText(0, -text.getLineHeight());
		}
		
		// we tell the contentByte, we've finished drawing text
		cb.endText();
	}

	@Override
	protected void writePicture(Picture pic) throws IOException {
		Image image = pic.getCroppedImage();
		try {
			PdfContentByte cb = _pdfWriter.getDirectContent();
			com.lowagie.text.Image iTextImage = com.lowagie.text.Image.getInstance(image, null);
			float angle = (float) (pic.getRotate() * Math.PI / 180);
			double sin = Math.sin(angle), cos = Math.cos(angle);
			int w = pic.getWidth(), h = pic.getHeight();
			iTextImage.setAbsolutePosition((float) (pic.getX() + (w - Math.abs(w * cos + h * sin)) / 2),
					(float) (_pdfWriter.getPageSize().getHeight() - pic.getY() - (h + Math.abs(w * sin + h * cos)) / 2));
			iTextImage.scalePercent(pic.getScale() * 100);
			iTextImage.setRotation(-angle);
			cb.addImage(iTextImage);
		} catch (DocumentException e) {
			e.printStackTrace();
		}
	}
	
	@Override
	protected void writeCircle(Circle circle) {
		PdfContentByte cb = _pdfWriter.getDirectContent();
		cb.circle(circle.getCenter().getX(), _pdfWriter.getPageSize().getHeight() - circle.getCenter().getY(), (float) circle.getRadius());
		cb.setColorFill(circle.getFillColor());
		if(circle.getThickness() > 0) {
			cb.setLineWidth(circle.getThickness());
			cb.setColorStroke(circle.getColor());
			cb.fillStroke();
		} else {
			cb.fill();
		}
	}
	
	private List<Line> seenLines = new LinkedList<Line>();
	@Override
	protected void writeLine(Line line) throws IOException {
		if(seenLines.contains(line)) {
			return;
		}
		PdfContentByte cb = _pdfWriter.getDirectContent();
		float height = _pdfWriter.getPageSize().getHeight();
		if(line.getStartItem().isEnclosed()) {
			seenLines.add(line);
			Line currentLine = line;
			Item currentItem = line.getStartItem();
			cb.moveTo(currentItem.getX(), height - currentItem.getY());
			for(;;) {
				currentItem = currentLine.getOppositeEnd(currentItem);
				cb.lineTo(currentItem.getX(), height - currentItem.getY());
				for(Line l : currentItem.getLines()) {
					if(l.equals(currentLine)) {
						continue;
					}
					currentLine = l;
					break;
				}
				if(currentLine == null || currentLine.equals(line)) {
					break;
				}
				seenLines.add(currentLine);
			}
			cb.closePath();
			cb.setColorFill(currentItem.getFillColor());
			if(currentItem.getThickness() > 0) {
				cb.setLineWidth(currentItem.getThickness());
				cb.setColorStroke(currentLine.getColor());
				cb.fillStroke();
			} else {
				cb.fill();
			}
		} else {
			for(Item l : line.getStartItem().getAllConnected()) {
				if(l instanceof Line && !seenLines.contains(l)) {
					seenLines.add((Line) l);
					Item start = ((Line) l).getStartItem();
					Item end = ((Line) l).getEndItem();
					cb.moveTo(start.getX(), height - start.getY());
					cb.lineTo(end.getX(), height - end.getY());
					cb.closePath();
					if(l.getThickness() >= 0) {
						cb.setLineWidth(l.getThickness());
						cb.setColorStroke(l.getColor());
						cb.stroke();
					}
					Polygon arrow = ((Line) l).getStartArrow();
					if(arrow != null) {
						cb.moveTo(arrow.xpoints[0], height - arrow.ypoints[0]);
						for(int i = 1; i < arrow.npoints; i++) {
							cb.lineTo(arrow.xpoints[i], height - arrow.ypoints[i]);
						}
						cb.closePath();
						cb.setColorFill(l.getColor());
						if(l.getThickness() > 0) {
    						cb.setLineWidth(l.getThickness());
    						cb.setColorStroke(l.getColor());
    						cb.stroke();
    					} else {
    						cb.fill();
    					}
					}
					arrow = ((Line) l).getEndArrow();
					if(arrow != null) {
						cb.moveTo(arrow.xpoints[0], height - arrow.ypoints[0]);
						for(int i = 1; i < arrow.npoints; i++) {
							cb.lineTo(arrow.xpoints[i], height - arrow.ypoints[i]);
						}
						cb.closePath();
						cb.setColorFill(l.getColor());
						if(l.getThickness() > 0) {
    						cb.setLineWidth(l.getThickness());
    						cb.setColorStroke(l.getColor());
    						cb.fillStroke();
    					} else {
    						cb.fill();
    					}
					}
				}
			}
		}
	}
	
	@Override
	protected Collection<Item> getItemsToWrite(Frame toWrite) {
		return toWrite.getVisibleItems();
	}
}
